<?php
namespace Opencart\Catalog\Controller\Extension\IdPaymentBPC\Payment;
use Opencart\Catalog\Model\Extension\IdPaymentBPC\Payment\Gateway;
include_once(DIR_EXTENSION . 'id_payment_bpc/system/library/Gateway.php');
class BPCPayment extends \Opencart\System\Engine\Controller
{
public function index(): string
{
$this->load->language('extension/id_payment_bpc/payment/bpc_payment');
if (isset($this->session->data['payment_method'])) {
$data['language'] = $this->config->get('config_language');
return $this->load->view('extension/id_payment_bpc/payment/bpc_payment', $data);
}
return '';
}
public function confirm(): void
{
$this->load->language('extension/id_payment_bpc/payment/bpc_payment');
$json = array();
if (isset($this->session->data['order_id'])) {
$order_id = $this->session->data['order_id'];
} else {
$order_id = 0;
}
$this->load->model('checkout/order');
$order_info = $this->model_checkout_order->getOrder($order_id);
if (!$order_info) {
$json['error']['warning'] = $this->language->get('error_order');
}
if (!$this->config->get('payment_bpc_payment_status') || !isset($this->session->data['payment_method']) || $this->session->data['payment_method']['code'] != 'bpc_payment.bpc_payment') {
$json['error']['warning'] = $this->language->get('error_payment_method');
}
if (!$json) {
/*
* Credit Card (bpc) charge code goes here
*/
$return_url = $this->url->link('extension/id_payment_bpc/payment/bpc_payment.comeback');
$Gateway = new Gateway($this->getPaymentParams());
$jsonParams = array(
'CMS' => 'OpenCart ' . VERSION,
'Module-Version' => $Gateway->module_version,
);
if (!empty($order_info['email'])) {
$jsonParams['email'] = $order_info['email'];
}
if (!empty($order_info['telephone'])) {
}
if ( $Gateway->enable_back_url_settings
&& !empty($this->config->get('payment_bpc_payment_backToShopUrl'))
) {
$jsonParams['backToShopUrl'] = $this->config->get('payment_bpc_payment_backToShopUrl');
}
$order_number = (int)$order_info['order_id'];
$amount = round($order_info['total'] * $order_info['currency_value'], 2) * 100;
$args = array(
'orderNumber' => $order_number . "_" . time(),
'amount' => $amount,
'returnUrl' => $return_url,
'jsonParams' => json_encode($jsonParams),
);
if (defined('BPC_BPC_MANDATORY_CURRENCY') && BPC_BPC_MANDATORY_CURRENCY === true) {
$numeric_code = $this->_get_numeric_currency_code($order_info['currency_code']);
if ($numeric_code !== null) {
$args['currency'] = $numeric_code;
}
}
if (!empty($order_info['customer_id'] && $order_info['customer_id'] > 0)) {
$client_email = !empty($order_info['email']) ? $order_info['email'] : "";
$args['clientId'] = md5($order_info['customer_id'] . $client_email . $order_info['store_url']);
}
if (defined('BPC_BPC_SEND_CLIENT_FULL_INFO') && BPC_BPC_SEND_CLIENT_FULL_INFO === true) {
$billingPayerData = $this->_getBillingPayerData($order_info);
if(!empty($billingPayerData)) {
$args['billingPayerData'] = json_encode($billingPayerData);
}
}
if (!empty($this->config->get('payment_bpc_payment_merchant_token'))) {
list($args['userName'], $args['password']) = explode(':', base64_decode($this->config->get('payment_bpc_payment_merchant_token')));
} else {
$args['userName'] = $this->config->get('payment_bpc_payment_merchant_login');
$args['password'] = $this->config->get('payment_bpc_payment_merchant_password');
}
if ($Gateway->enable_cart_options) {
$orderBundle = array();
$orderBundle['customerDetails'] = array(
'email' => $order_info['email'],
);
foreach ($this->cart->getProducts() as $product) {
$product_taxSum = $this->tax->getTax($product['price'], $product['tax_class_id']);
$product_amount = (round($product['price'] + $product_taxSum, 2)) * $product['quantity'];
if ($product['tax_class_id'] != 0) {
$item_rate = $product_taxSum / $product['price'] * 100;
switch ($item_rate) {
case 20:
$tax_type = 6;
break;
case 18:
$tax_type = 3;
break;
case 10:
$tax_type = 2;
break;
case 0:
$tax_type = 1;
break;
default:
$tax_type = $this->config->get('payment_bpc_payment_tax_type');
}
}
$product_data = array(
'positionId' => $product['cart_id'],
'name' => $product['name'],
'quantity' => array(
'value' => $product['quantity'],
'measure' => $Gateway->getDefaultMeasurement(),
),
'itemAmount' => $product_amount * 100,
'itemCode' => $product['product_id'] . "_" . $product['cart_id'], //fix by PLUG-1740, PLUG-2620
'tax' => array(
'taxType' => $tax_type,
'taxSum' => $product_taxSum * 100
),
'itemPrice' => round($product['price'] + $product_taxSum, 2) * 100,
);
$attributes = array();
$attributes[] = array(
"name" => "paymentMethod",
"value" => $Gateway->getPaymentMethodType()
);
$attributes[] = array(
"name" => "paymentObject",
"value" => $Gateway->getPaymentObjectType()
);
$product_data['itemAttributes']['attributes'] = $attributes;
$orderBundle['cartItems']['items'][] = $product_data;
}
if (isset($this->session->data['shipping_method']['cost']) && $this->session->data['shipping_method']['cost'] > 0) {
$delivery['positionId'] = 'delivery';
$delivery['name'] = $this->session->data['shipping_method']['name']; //ex.title
$delivery['itemAmount'] = $this->session->data['shipping_method']['cost'] * 100;
$delivery['quantity']['value'] = 1;
$delivery['quantity']['measure'] = $Gateway->getDefaultMeasurement(); //todo?
$delivery['itemCode'] = $this->session->data['shipping_method']['code'];
$delivery['tax']['taxType'] = $this->config->get('payment_bpc_payment_tax_type');
$delivery['tax']['taxSum'] = 0;
$delivery['itemPrice'] = $this->session->data['shipping_method']['cost'] * 100;
$attributes = array();
$attributes[] = array(
"name" => "paymentMethod",
"value" => $Gateway->getPaymentMethodType(true)
);
$attributes[] = array(
"name" => "paymentObject",
"value" => 4
);
$delivery['itemAttributes']['attributes'] = $attributes;
$orderBundle['cartItems']['items'][] = $delivery;
}
if (isset($this->session->data['vouchers']) && count($this->session->data['vouchers']) > 0) {
foreach ($this->session->data['vouchers'] as $key => $voucher) {
$itemVoucher = array(
'positionId' => 'voucher_' . $key,
'name' => $voucher['description'],
'itemAmount' => $voucher['amount'] * 100,
'quantity' => array(
'value' => 1,
'measure' => $Gateway->getDefaultMeasurement(),
),
'itemCode' => 'voucher_' . $key,
'tax' => array(
'taxSum' => 0,
),
'itemPrice' => $voucher['amount'] * 100
);
$attributes = array();
$attributes[] = array(
"name" => "paymentMethod",
"value" => $Gateway->getPaymentMethodType(),
);
$attributes[] = array(
"name" => "paymentObject",
"value" => 1
);
$itemVoucher['itemAttributes']['attributes'] = $attributes;
$orderBundle['cartItems']['items'][] = $itemVoucher;
}
}
$orderBundle['orderCreationDate'] = date('c');
$args['taxSystem'] = $Gateway->taxSystem;
$args['orderBundle'] = json_encode($orderBundle);
}
$headers = array(
'CMS: OpenCart ' . VERSION,
'Module-Version: ' . $Gateway->module_version,
);
$action_adr = $Gateway->test_mode ? $Gateway->getTestUrl() : $Gateway->getProdUrl();
$action_adr .= $Gateway->stage_mode != 'one' ? 'registerPreAuth.do' : 'register.do';
$response = $Gateway->_sendGatewayData(http_build_query($args, '', '&'), $action_adr, $headers);
$Gateway->writeLog("REQUEST:\n". $action_adr . "\n: " . print_r($args, true) . "\nRESPONSE:\n" . $response);
$response = json_decode($response, true);
if (empty($response['errorCode'])) {
$json['redirect'] = $response['formUrl'];
} else {
$json['error']['warning'] = $response['errorCode'] . " " . $response['errorMessage'];
}
}
$this->response->addHeader('Content-Type: application/json');
$this->response->setOutput(json_encode($json));
}
public function stored(): void
{
$this->load->language('extension/id_payment_bpc/payment/bpc_payment');
$json = [];
if (isset($this->session->data['order_id'])) {
$order_id = $this->session->data['order_id'];
} else {
$order_id = 0;
}
if (isset($this->session->data['payment_method'])) {
$payment = explode('.', $this->session->data['payment_method']['code']);
} else {
$payment = [];
}
if (isset($payment[0])) {
$payment_method = $payment[0];
} else {
$payment_method = '';
}
if (isset($payment[1])) {
$bpc_payment_id = $payment[1];
} else {
$bpc_payment_id = 0;
}
$this->load->model('checkout/order');
$order_info = $this->model_checkout_order->getOrder($order_id);
if (!$order_info) {
$json['error']['warning'] = $this->language->get('error_order');
}
if (!$this->customer->isLogged()) {
$json['error']['warning'] = $this->language->get('error_login');
}
if (!$this->config->get('payment_bpc_payment_status') || $payment_method != 'bpc_payment') {
$json['error']['warning'] = $this->language->get('error_payment_method');
}
$this->load->model('extension/id_payment_bpc/payment/bpc_payment');
$bpc_payment_info = $this->model_extension_id_payment_bpc_payment_bpc_payment->getBPCPayment($this->customer->getId(), $bpc_payment_id);
if (!$bpc_payment_info) {
$json['error']['warning'] = $this->language->get('error_bpc_payment');
}
if (!$json) {
/*
*
* Credit Card (bpc) validation code goes here
*
*/
$response = $this->model_extension_id_payment_bpc_payment_bpc_payment->charge($this->customer->getId(), $this->session->data['order_id'], $order_info['total'], $bpc_payment_id);
if ($response) {
$this->load->model('checkout/order');
$this->model_checkout_order->addHistory($this->session->data['order_id'], $this->config->get('payment_bpc_payment_approved_status_id'), '', true);
$json['redirect'] = $this->url->link('checkout/success', 'language=' . $this->config->get('config_language'), true);
} else {
$this->load->model('checkout/order');
$this->model_checkout_order->addHistory($this->session->data['order_id'], $this->config->get('payment_bpc_payment_failed_status_id'), '', true);
$json['redirect'] = $this->url->link('checkout/failure', 'language=' . $this->config->get('config_language'), true);
}
}
$this->response->addHeader('Content-Type: application/json');
$this->response->setOutput(json_encode($json));
}
public function delete(): void
{
$this->load->language('extension/id_payment_bpc/payment/bpc_payment');
$json = [];
if (isset($this->request->get['bpc_payment_id'])) {
$bpc_payment_id = (int)$this->request->get['bpc_payment_id'];
} else {
$bpc_payment_id = 0;
}
if (!$this->customer->isLogged()) {
$json['error'] = $this->language->get('error_logged');
}
$this->load->model('extension/id_payment_bpc/payment/bpc_payment');
$bpc_payment_info = $this->model_extension_id_payment_bpc_payment_bpc_payment->getBPCPayment($this->customer->getId(), $bpc_payment_id);
if (!$bpc_payment_info) {
$json['error'] = $this->language->get('error_bpc_payment');
}
if (!$json) {
$this->model_extension_id_payment_bpc_payment_bpc_payment->deleteBPCPayment($this->customer->getId(), $bpc_payment_id);
$json['success'] = $this->language->get('text_delete');
unset($this->session->data['shipping_method']);
unset($this->session->data['shipping_methods']);
unset($this->session->data['payment_method']);
unset($this->session->data['payment_methods']);
}
$this->response->addHeader('Content-Type: application/json');
$this->response->setOutput(json_encode($json));
}
public function comeback(): void
{
if (isset($this->request->get['orderId'])) {
$order_id = $this->request->get['orderId'];
} else {
die('Illegal Access');
}
$Gateway = new Gateway($this->getPaymentParams());
if (!empty($this->config->get('payment_bpc_payment_merchant_token'))) {
list($args['userName'], $args['password']) = explode(':', base64_decode($this->config->get('payment_bpc_payment_merchant_token')));
} else {
$args['userName'] = $this->config->get('payment_bpc_payment_merchant_login');
$args['password'] = $this->config->get('payment_bpc_payment_merchant_password');
}
$response = $Gateway->_getGatewayOrderStatus($args, $order_id);
$response = json_decode($response, true);
$ex = explode("_", $response['orderNumber']);
$order_number = $ex[0];
$this->load->model('checkout/order');
$order_info = $this->model_checkout_order->getOrder($order_number);
if ($order_info) {
if (($response['errorCode'] == 0) && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {
if ($Gateway->allowCallbacks == false) {
$this->_storeGatewayOrderData($order_id, $order_info, $response);
$comment = "Incoming from gateway by ReturnUrl";
$this->model_checkout_order->addHistory($order_number, $this->config->get('payment_bpc_payment_approved_status_id'), $comment, false);
}
$this->response->redirect($this->url->link('checkout/success', '', true));
} else {
$this->response->redirect($this->url->link('checkout/failure', '', true));
}
}
}
public function callback(): void
{
if (isset($this->request->get['mdOrder'])) {
$order_id = $this->request->get['mdOrder'];
} else {
die('Illegal Access');
}
$Gateway = new Gateway($this->getPaymentParams());
if (!empty($this->config->get('payment_bpc_payment_merchant_token'))) {
list($args['userName'], $args['password']) = explode(':', base64_decode($this->config->get('payment_bpc_payment_merchant_token')));
} else {
$args['userName'] = $this->config->get('payment_bpc_payment_merchant_login');
$args['password'] = $this->config->get('payment_bpc_payment_merchant_password');
}
$response = $Gateway->_getGatewayOrderStatus($args, $order_id);
$response = json_decode($response, true);
$ex = explode("_", $response['orderNumber']);
$order_number = $ex[0];
$this->load->model('checkout/order');
$order_info = $this->model_checkout_order->getOrder($order_number);
if ($order_info) {
if (($response['errorCode'] == 0) && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {
$this->_storeGatewayOrderData($order_id, $order_info, $response);
$comment = "Incoming callback";
$this->model_checkout_order->addHistory($order_number, $this->config->get('payment_bpc_payment_approved_status_id'), $comment, false);
} else {
}
}
}
private function getPaymentParams() {
return [
'test_mode' => $this->config->get('payment_bpc_payment_test_mode'),
'stage_mode' => $this->config->get('payment_bpc_payment_stage_mode'),
'paymentMethodType' => $this->config->get('payment_bpc_payment_method'),
'paymentMethodTypeDelivery' => $this->config->get('payment_bpc_payment_method_delivery'),
'paymentObjectType' => $this->config->get('payment_bpc_payment_object'),
'versionFfd' => $this->config->get('payment_bpc_payment_version_ffd'),
'taxSystem' => $this->config->get('payment_bpc_payment_tax_system'),
'taxType' => $this->config->get('payment_bpc_payment_tax_type')
];
}
public function _storeGatewayOrderData($order_id, $order_info, $response) {
$this->load->model('extension/id_payment_bpc/payment/bpc_payment');
$data = array(
'order_id' => (int)$order_info['order_id'],
'gateway_order_reference' => $order_id,
'currency' => $response['currency'],
'order_amount' => $response['amount'],
'order_amount_deposited' => $response['amount'],
'status_deposited' => $response['orderStatus'] === 1 ? 0 : 1, //todo
);
$this->model_extension_id_payment_bpc_payment_bpc_payment->storeGatewayOrder($data);
}
private function _getBillingPayerData($orderData) {
$billingPayerData = array(
'billingCity' => '',
'billingCountry' => '',
'billingAddressLine1' => '',
'billingAddressLine2' => '',
'billingAddressLine3' => '',
'billingPostalCode' => '',
'billingState' => ''
);
if (isset($orderData['payment_address_id']) && $orderData['payment_address_id'] != 0) {
$billingPayerData['billingCity'] = $orderData['payment_city'];
$billingPayerData['billingCountry'] = $orderData['payment_iso_code_2'];//$orderData['payment_country'];
$billingPayerData['billingAddressLine1'] = $orderData['payment_address_1'];
$billingPayerData['billingAddressLine2'] = $orderData['payment_address_2'];
$billingPayerData['billingPostalCode'] = $orderData['payment_postcode'];
$billingPayerData['billingState'] = $orderData['payment_zone'];
} else {
$billingPayerData['billingCity'] = $orderData['shipping_city'];
$billingPayerData['billingCountry'] = $orderData['shipping_iso_code_2'];//$orderData['shipping_country'];
$billingPayerData['billingAddressLine1'] = $orderData['shipping_address_1'];
$billingPayerData['billingAddressLine2'] = $orderData['shipping_address_2'];
$billingPayerData['billingPostalCode'] = $orderData['shipping_postcode'];
$billingPayerData['billingState'] = $orderData['shipping_zone'];
}
return $billingPayerData;
}
function _get_numeric_currency_code($currency_code) {
$currency_codes = array(
'BYN' => '933',
'BHD' => '048',
'BYR' => '974',
'CAD' => '124',
'CNY' => '156',
'EUR' => '978',
'GBP' => '826',
'HKD' => '344',
'HUF' => '348',
'ILS' => '376',
'JPY' => '392',
'KGS' => '417',
'KRW' => '410',
'KZT' => '398',
'MDL' => '498',
'MYR' => '458',
'OMR' => '512',
'PHP' => '608',
'RON' => '946',
'RUB' => '643',
'RUR' => '810',
'SGD' => '702',
'UAH' => '980',
'USD' => '840',
'NGN' => '566',
'MZN' => '943',
'BGN' => '975',
'BZD' => '084',
'GHS' => '936',
'GNF' => '324',
'XOF' => '952',
'PLN' => '985',
'LSL' => '426',
'TZS' => '834',
'NZD' => '554',
'KHR' => '116',
'TRY' => '949',
'AMD' => '051',
'SAR' => '682',
'AED' => '784',
'COP' => '170',
'AUD' => '036',
'IDR' => '360',
'KWD' => '414',
'JOD' => '400',
'INR' => '356'
);
return isset($currency_codes[$currency_code]) ? $currency_codes[$currency_code] : null;
}
}